"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable import/no-extraneous-dependencies */
const _ = require("lodash");
function formatProperty(value, key, array = false) {
    let name;
    const desc = { "value": value[key] };
    const str = formatValue(desc.value);
    if (array && key.match(/^\d+$/)) {
        return str;
    }
    name = JSON.stringify(`${key}`);
    if (name.match(/^"([a-zA-Z_]\w*)"$/)) {
        name = name.substr(1, name.length - 2);
    }
    else {
        // eslint-disable-next-line lodash/chaining
        name = _.replace(_.replace(_.replace(name, /'/g, "\\'"), /\\"/g, '"'), /(^"|"$)/g, "'");
    }
    return `${name}: ${str}`;
}
function formatValue(value) {
    // Provide a hook for user-specified inspect functions.
    // Check that value is an object with an inspect function on it
    if (value && _.isFunction(value.inspect) &&
        // Filter out the util module, it's inspect function is special
        value.inspect !== inspect &&
        // Also filter out any prototype objects using the circular check.
        !(value.constructor && value.constructor.prototype === value)) {
        return value.inspect();
    }
    // Primitive types cannot have properties
    let primitive;
    if (_.isUndefined(value))
        primitive = "undefined";
    if (_.isString(value)) {
        // eslint-disable-next-line lodash/chaining
        const simple = `'${_.replace(_.replace(_.replace(JSON.stringify(value), /^"|"$/g, ""), /'/g, "\\'"), /\\"/g, '"')}'`;
        primitive = simple;
    }
    if (_.isNumber(value))
        primitive = `${value}`;
    if (_.isBoolean(value))
        primitive = `${value}`;
    // For some reason typeof null is "object", so special case here.
    if (_.isNull(value))
        primitive = "null";
    if (primitive)
        return primitive;
    // Look up the keys of the object.
    const keys = _.keys(value);
    // Some type of object without properties can be shortcutted.
    if (keys.length === 0) {
        if (_.isFunction(value)) {
            let name = "";
            if (value.name)
                name = `: ${value.name}`;
            return `[Function${name}]`;
        }
        if (_.isRegExp(value)) {
            return RegExp.prototype.toString.call(value);
        }
        if (_.isDate(value)) {
            return Date.prototype.toString.call(value);
        }
        if (_.isError(value)) {
            return `[${Error.prototype.toString.call(value)}]`;
        }
    }
    const base = "";
    let array = false;
    let braces = ["{", "}"];
    // Make Array say that they are Array
    if (_.isArray(value)) {
        array = true;
        braces = ["[", "]"];
    }
    if (keys.length === 0 && (!array || value.length === 0)) {
        return braces[0] + base + braces[1];
    }
    let output;
    if (array) {
        output = [];
        for (let i = 0, l = value.length; i < l; ++i) {
            output.push(formatProperty(value, String(i), true));
        }
    }
    else {
        output = _.map(keys, (key) => formatProperty(value, key));
    }
    return `${braces[0]}${base} ${output.join(", ")} ${braces[1]}`;
}
function inspect(obj) {
    return formatValue(obj);
}
exports.default = inspect;
//# sourceMappingURL=inspect.js.map