"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
//NEED
function deepEqual(obj, ref, options = {
        "part": false,
        "prototype": true
    }, seen = []) {
    const type = typeof obj;
    if (type !== typeof ref) {
        return false;
    }
    if (type !== "object" ||
        obj === null ||
        ref === null) {
        if (obj === ref) {
            return obj !== 0 || 1 / obj === 1 / ref;
        }
        return obj !== obj && ref !== ref;
    }
    if (_.includes(seen, obj)) {
        return true;
    }
    seen.push(obj);
    if (_.isArray(obj)) {
        if (!_.isArray(ref)) {
            return false;
        }
        if (!options.part && obj.length !== ref.length) {
            return false;
        }
        for (let i = 0; i < obj.length; ++i) {
            if (options.part) {
                let found = false;
                for (let j = 0; j < ref.length; ++j) {
                    if (deepEqual(obj[i], ref[j], options)) {
                        found = true;
                        break;
                    }
                }
                return found;
            }
            if (!deepEqual(obj[i], ref[i], options)) {
                return false;
            }
        }
        return true;
    }
    if (Buffer.isBuffer(obj)) {
        if (!Buffer.isBuffer(ref)) {
            return false;
        }
        if (obj.length !== ref.length) {
            return false;
        }
        for (let i = 0; i < obj.length; ++i) {
            if (obj[i] !== ref[i]) {
                return false;
            }
        }
        return true;
    }
    if (obj instanceof Date) {
        return (ref instanceof Date && obj.getTime() === ref.getTime());
    }
    if (obj instanceof RegExp) {
        return (ref instanceof RegExp && obj.toString() === ref.toString());
    }
    if (options.prototype) {
        if (Object.getPrototypeOf(obj) !== Object.getPrototypeOf(ref)) {
            return false;
        }
    }
    const keys = Object.getOwnPropertyNames(obj);
    if (!options.part && keys.length !== Object.getOwnPropertyNames(ref).length) {
        return false;
    }
    for (let i = 0; i < keys.length; ++i) {
        const key = keys[i];
        if (!deepEqual(obj[key], ref[key], options, seen)) {
            return false;
        }
    }
    return true;
}
exports.deepEqual = deepEqual;
// Test if the reference contains the values
// options deep, once, only, part
// NEED
function contain(ref, values, options = {}) {
    /*
        string -> string(s)
        array -> item(s)
        object -> key(s)
        object -> object (key:value)
    */
    let valuePairs = null;
    if (typeof ref === "object" &&
        typeof values === "object" &&
        !_.isArray(ref) &&
        !_.isArray(values)) {
        valuePairs = values;
        values = _.keys(values);
    }
    else {
        values = [].concat(values);
    }
    let compare;
    let compareFlags;
    if (options.deep) {
        compare = deepEqual;
        const hasOnly = {}.hasOwnProperty.call(options, "only");
        const hasPart = {}.hasOwnProperty.call(options, "part");
        compareFlags = {
            "prototype": false,
            "part": false
        };
        if (hasOnly) {
            compareFlags.prototype = options.only || false;
            compareFlags.part = !options.only;
        }
        else {
            if (hasPart) {
                compareFlags.prototype = !options.part;
                compareFlags.part = options.part || false;
            }
            else {
                compareFlags.prototype = false;
                compareFlags.part = true;
            }
        }
    }
    else {
        compare = (a, b) => a === b;
    }
    let misses = false;
    const matches = new Array(values.length);
    for (let i = 0; i < matches.length; ++i) {
        matches[i] = 0;
    }
    if (typeof ref === "string") {
        let pattern = "(";
        for (let i = 0; i < values.length; ++i) {
            const value = values[i];
            if (i) {
                pattern += "|";
            }
            else {
                pattern += "";
            }
            pattern += _.replace(value, /[\^$.*+\-?=!:|\\/()[\]{},]/g, "\\$&");
        }
        const regex = new RegExp(`${pattern})`, "g");
        const leftovers = _.replace(ref, regex, ($0, $1) => {
            const index = values.indexOf($1);
            ++matches[index];
            // Remove from string
            return "";
        });
        misses = !!leftovers;
    }
    else if (_.isArray(ref)) {
        for (let i = 0; i < ref.length; ++i) {
            let matched = false;
            for (let j = 0; j < values.length && matched === false; ++j) {
                matched = compare(values[j], ref[i], compareFlags) && j;
            }
            if (matched !== false) {
                ++matches[matched];
            }
            else {
                misses = true;
            }
        }
    }
    else {
        const keys = Object.getOwnPropertyNames(ref);
        for (let i = 0; i < keys.length; ++i) {
            const key = keys[i];
            const pos = values.indexOf(key);
            if (pos !== -1) {
                if (valuePairs &&
                    !compare(valuePairs[key], ref[key], compareFlags)) {
                    return false;
                }
                ++matches[pos];
            }
            else {
                misses = true;
            }
        }
    }
    let result = false;
    for (let i = 0; i < matches.length; ++i) {
        result = result || !!matches[i];
        if ((options.once && matches[i] > 1) ||
            (!options.part && !matches[i])) {
            return false;
        }
    }
    if (options.only && misses) {
        return false;
    }
    return result;
}
exports.contain = contain;
// NEED
function wait(timeout) {
    return new Promise((resolve) => setTimeout(resolve, timeout));
}
exports.wait = wait;
//# sourceMappingURL=utils.js.map